/*
 * AddInInfo
 *
 * Author: J. Steingraeber
 * Date: 2001/06/10
 *
 * Released to the public domain
 */

#include <stdrom.h>
#include "define.h"
#include "libc.h"
#include "l_define.h"
#include "l_libc.h"

#include "AddInInfo.h"

/*
 * SwitchBank
 *
 * makes RAM or FLASH memory accessible at certain bank
 *
 * addr A31-A16 of source address (must be even)
 *      use 0000 - 00FE for RAM (only 0000 will work)
 *      use 0100 - 010E for program flash (1st MB, PVOS)
 *      use 0110 - 011E for program flash (2nd MB, Add-In)
 *      use 0120 - 013E for data flash (4MB data)
 *
 * bank destination bank
 *      use 0 for address space 0000:0000 (forbidden)
 *      use 1 for address space 2000:0000
 *      use 2 for address space 4000:0000
 *      use 3 for address space 6000:0000
 *      use 4 for address space 8000:0000 (forbidden)
 *      use 5 for address space A000:0000
 *      use 6 for address space C000:0000 (forbidden)
 *
 * remarks:
 *   you can not switch bank 0, 4 and 6, because that will crash your
 *   PV; bank 0 contains the interrupt vectors, and every program
 *   expects to find RAM here; bank 4 contains your Add-In, so
 *   switching this would make your Add-In vanishing; bank 6 contains
 *   the BIOS, your PV can not operate without it
 *
 * examples:
 *
 *   SwitchBank(0x0000,3);
 *     will make 128kB RAM accessible at address 6000:0000 so RAM can
 *     be accessed at 0000:0000 and 6000:0000 in the same way (which
 *     is possible but stupid, you still have only 128kB of RAM)
 *
 *   SwitchBank(0x0104,5);
 *     will make Fonts accessible at address A000:0000 and Graphics
 *     accessible at B000:0000
 *
 *   SwitchBank(0x0120,1);
 *     will make first 128kB of data flash accessible at 2000:0000
 *
 */
void _asm_switchbank(char *,int,int);
#define SwitchBank(addr,bank) _asm_switchbank("\n MOV BH,AL\n MOV AL,AH\n MOV AH,BL\n OR AH,80h\n INT 0C8h\n MOV AL,BH\n MOV AH,BL\n INT 0C8h\n",addr,bank);

void far startaddin(unsigned char StartSegHighByte,
                    unsigned int FirstBlock, int n);

int far deladdin(unsigned int wSeg, unsigned int wOffs);

void far defrag();

void StartAddIn(unsigned long lLinAddr)
{
  unsigned int far * p;
  unsigned int wSeg, wOffs;
  unsigned long far * pDW;
  unsigned long lLength;
  
      /*
       * determine length of Add-In in 16kB Blocks
       */
  
      /* make selected memory visible at 2000:0000 */
  SwitchBank((((lLinAddr) >> 16) & 0xFFFE),1);

  wSeg = 0x2000;
  wOffs = lLinAddr & 0xFFFF;
  if (lLinAddr & 0x00010000)
  {
    wSeg = 0x3000;
  }

  pDW = MK_FP(wSeg,wOffs);
  lLength = pDW[9]; /* get length from Add-In header */

      /* adjust for block count */
  lLength += 0x3FFF;
  lLength = lLength >> 14; /* -> 16kB blocks */
  
  lLinAddr = lLinAddr >> 14;
  lLinAddr = lLinAddr & 0xFFFF;

  p = MK_FP(0xC400,0x115C);
  if (p[0] == 0x6682)
  {
        /* S-Series */
    p = MK_FP(0x0000,0x6682);
    p[0] = lLinAddr;
    p[1] = (word)lLength;
  }
  else
  {
    p = MK_FP(0xC400,0x10DA);
    if (p[0] == 0x6ABA)
    {
          /* X-Series */
      p = MK_FP(0x0000,0x6ABA);
      p[0] = lLinAddr;
      p[1] = (word)lLength;
    }
    else
    {
      p = MK_FP(0xC400,0x10F8);
      if (p[0] == 0x6682)
      {
            /* 750-Series */
        p = MK_FP(0x0000,0x6682);
        p[0] = lLinAddr;
        p[1] = (word)lLength;
      }
      else
      {
        p = MK_FP(0xC400,0x10AA);
        if (p[0] == 0x65E6)
        {
              /* S460/660 */
          p = MK_FP(0x0000,0x65E6);
          p[0] = lLinAddr;
          p[1] = (word)lLength;
        }
        else
        {
              /* unknown model */
          LibPutMsgDlg("WARNING!\r\rUnrecognized PVOS!\rUse on your own risk!\rBetter switch off now!");
        }
      }
    }
  }
  
  lLinAddr = lLinAddr | 0x8000;
  startaddin(0x80,lLinAddr,(word)lLength);
}

int CanDelAddIn(int iNr)
{
  unsigned int far * p;
  unsigned long LinAddr, lTemp;

  p = MK_FP(0xC400,0x10DA);
  if (((iNr % 4) != 0) /* cannot delete packed AddIn */
      || ((p[0] != 0x6ABA) && (iNr == 0)))
  {
        /* not X-model => don't delete build-in Add-In */
    return 0;
  }

  p = MK_FP(0xC400,0x115C);
  if (p[0] == 0x6682)
  {
        /* S-Series */
    p = MK_FP(0x0000,0x6682);
  }
  else
  {
    p = MK_FP(0xC400,0x10DA);
    if (p[0] == 0x6ABA)
    {
          /* X-Series */
      p = MK_FP(0x0000,0x6ABA);
    }
    else
    {
      p = MK_FP(0xC400,0x10F8);
      if (p[0] == 0x6682)
      {
            /* 750-Series */
        p = MK_FP(0x0000,0x6682);
      }
      else
      {
        p = MK_FP(0xC400,0x10AA);
        if (p[0] == 0x65E6)
        {
              /* S460/660 */
          p = MK_FP(0x0000,0x65E6);
        }
        else
        {
              /* unknown PVOS */
          return 0;
        }
      }
    }
  }
  
  lTemp = iNr;
  lTemp = lTemp << 14; /* each slot has a size of 16kB */
  LinAddr = 0x01100000; /* Address of first Add-In for X-Models */
  LinAddr += lTemp;
  LinAddr = LinAddr >> 14;
  LinAddr = LinAddr & 0xFFFF;

  if (LinAddr == p[0])
  {
        /* don't delete actual running Add-In */
    return 0;
  }
  
  return 1;
}

int DelAddIn(int iNr)
{
  unsigned int far * p;
  unsigned int wSeg, wOffs;
  unsigned long LinAddr, lTemp;
  int i;
  unsigned char far * pByte;
  char name[16];
  
  p = MK_FP(0xC400,0x10DA);
  if ((p[0] != 0x6ABA) && (iNr == 0))
  {
        /* not X-model => don't delete build-in Add-In */
    return 0;
  }

      /* calculate start address of requested Add-In */
  lTemp = iNr;
  lTemp = lTemp << 14; /* each slot has a size of 16kB */
  LinAddr = 0x01100000; /* Address of first Add-In for X-Models */
  LinAddr += lTemp;

      /* make selected memory visible at 2000:0000 */
  SwitchBank((((LinAddr) >> 16) & 0xFFFE),1);

  wSeg = 0x2000;
  wOffs = LinAddr & 0xFFFF;
  if (LinAddr & 0x00010000)
  {
    wSeg = 0x3000;
  }

  if (deladdin(wSeg,wOffs))
  {
        /* Add-In is marked as deleted now, update Menu structure */
        /* You must call LibSaveSysRam() to make this permanent */

        /* search deleted Add-In in Menu structure */
    pByte = MK_FP(0x1840,0x0B71);
    i = 0;
    while (i < 24)
    {
      if (pByte[5*i+2] == IB_MADDIN)
      {
            /* Add-In found in, try to get it's name */
        if (LibGetProgramName(name,IB_MADDIN,pByte[5*i+1]) == FALSE)
        {
              /* this one is the delted one */
          break;
        }
      }
      ++i;
    }

    if (i < 24)
    {
      while (i < 23)
      {
            /* shift info for one Add-In */
        pByte[5*i+0] = pByte[5*i+5+0];
        pByte[5*i+1] = pByte[5*i+5+1];
        pByte[5*i+2] = pByte[5*i+5+2];
        pByte[5*i+3] = pByte[5*i+5+3];
        pByte[5*i+4] = pByte[5*i+5+4];
        ++i;
      }
          /* mark last Add-In as unused */
      pByte[5*i+0] = 0xFF;
      pByte[5*i+1] = 0xFF;
      pByte[5*i+2] = 0xFF;
      pByte[5*i+3] = 0xFF;
      pByte[5*i+4] = 0xFF;
    }
      
    return 1;
  }

  return 0;
}


void DefragAddInMem()
{
  defrag();
}

int GetAddInInfo(int iNr, AddInInfo far * pInfo)
{
  char far * p;
  int i;
  unsigned int n;
  unsigned long LinAddr, lTemp;
  unsigned long lIcon;
  unsigned int wSeg, wOffs;
  unsigned int far * pIconDim;
  
      /* do some sanity checks */
  if (pInfo == NULL)
  {
    return 0;
  }

      /* calculate start address of requested Add-In */
  lTemp = iNr;
  lTemp = lTemp << 14; /* each slot has a size of 16kB */
  LinAddr = 0x01100000; /* Address of first Add-In for X-Models */
  LinAddr += lTemp;

  pInfo->address = LinAddr;
  
      /* make selected memory visible at 2000:0000 */
  SwitchBank((((LinAddr) >> 16) & 0xFFFE),1);

  wSeg = 0x2000;
  wOffs = LinAddr & 0xFFFF;
  if (LinAddr & 0x00010000)
  {
    wSeg = 0x3000;
  }
      
  p = MK_FP(wSeg,wOffs);

      /* test for Add-In header */
  if ((far_strncmp(&p[2],"CASIO",5) == 0)
      && (far_strncmp(&p[8],"Z486",4) == 0))
  {
        /* Add-In found, get header information */
    pInfo->flags = 0;
    if (p[1] == 0)
    {
      pInfo->flags |= ADDININFO_DELETED;
    }
    if ((iNr % 4) != 0)
    {
      pInfo->flags |= ADDININFO_PACKED;
    }

    far_strncpy(pInfo->name,&p[20],16);
    pInfo->name[15] = 0;
    
    pInfo->size = ((unsigned long far *)p)[9];
    pInfo->orgsize = pInfo->size;
    
    n = 0;
    for (i=0;i<4;++i)
    {
      n = (n * 10) + p[40+i] - '0';
    }
    pInfo->date_year = n;
    
    n = 0;
    for (i=0;i<2;++i)
    {
      n = (n * 10) + p[44+i] - '0';
    }
    pInfo->date_month = n;
    
    n = 0;
    for (i=0;i<2;++i)
    {
      n = (n * 10) + p[46+i] - '0';
    }
    pInfo->date_day = n;
    
    n = 0;
    for (i=0;i<2;++i)
    {
      n = (n * 10) + p[48+i] - '0';
    }
    pInfo->date_hour = n;
    
    n = 0;
    for (i=0;i<2;++i)
    {
      n = (n * 10) + p[50+i] - '0';
    }
    pInfo->date_minute = n;
    
    n = 0;
    for (i=0;i<4;++i)
    {
      n = (n * 10) + p[52+i] - '0';
    }
    pInfo->version = n;
    
    n = 0;
    for (i=0;i<4;++i)
    {
      n = (n * 10) + p[56+i] - '0';
    }
    pInfo->lib_year = n;
    
    n = 0;
    for (i=0;i<2;++i)
    {
      n = (n * 10) + p[60+i] - '0';
    }
    pInfo->lib_month = n;
    
    n = 0;
    for (i=0;i<2;++i)
    {
      n = (n * 10) + p[62+i] - '0';
    }
    pInfo->lib_day = n;
    
    n = 0;
    for (i=0;i<2;++i)
    {
      n = (n * 10) + p[64+i] - '0';
    }
    pInfo->lib_hour = n;
    
    n = 0;
    for (i=0;i<2;++i)
    {
      n = (n * 10) + p[66+i] - '0';
    }
    pInfo->lib_minute = n;
    
    n = 0;
    for (i=0;i<4;++i)
    {
      n = (n * 10) + p[68+i] - '0';
    }
    pInfo->lib_version = n;

        /* get comment */
    far_strncpy(pInfo->comment,&p[0x50],MAXCOMMENTLENGTH);
    pInfo->comment[MAXCOMMENTLENGTH-1] = 0;
    if (pInfo->comment[0] == 0xFF)
    {
      pInfo->comment[0] = 0x00;
    }
    
        /* calculate address of menuicon */
    lTemp = ((unsigned long far *)p)[18];
    lTemp += LinAddr;

        /* calculate address of listicon */
    lIcon = ((unsigned long far *)p)[19];
    pInfo->orgsize = lIcon + 84;
    lIcon += LinAddr;

        /* make  memory for menuicon visible at 2000:0000 */
    SwitchBank((((lTemp) >> 16) & 0xFFFE),1);
    
    wSeg = 0x2000;
    wOffs = lTemp & 0xFFFF;
    if (lTemp & 0x00010000)
    {
      wSeg = 0x3000;
    }
    
    p = MK_FP(wSeg,wOffs);

    far_memcpy(pInfo->menuicon,p,176);
        /* force standard dimension */
    pIconDim = (unsigned int far *) &pInfo->menuicon[0];
    if (pIconDim[0] > 0x2D)
    {
      *pIconDim = 0x2D;
    }
    if (pIconDim[1] > 0x1C)
    {
      *pIconDim = 0x1C;
    }
    wOffs += 176;
    if ((wOffs != 0) && (wOffs < 176))
    {
          /* 64kB overflow */
      wSeg += 0x1000;
      if (wSeg == 0x4000)
      {
            /* correct bank */
        lTemp += 0x4000;
        SwitchBank((((lTemp) >> 16) & 0xFFFE),1);
        wSeg = 0x2000;
      }
      p = MK_FP(wSeg,0);
      far_memcpy(&(pInfo->menuicon[176-wOffs]),p,wOffs);
    }
    
        /* make  memory for menuicon visible at 2000:0000 */
    SwitchBank((((lIcon) >> 16) & 0xFFFE),1);
    
    wSeg = 0x2000;
    wOffs = lIcon & 0xFFFF;
    if (lIcon & 0x00010000)
    {
      wSeg = 0x3000;
    }
    
    p = MK_FP(wSeg,wOffs);

    far_memcpy(pInfo->listicon,p,84);
        /* force standard dimension */
    pIconDim = (unsigned int far *) &pInfo->listicon[0];
    if (pIconDim[0] > 0x1B)
    {
      *pIconDim = 0x1B;
    }
    if (pIconDim[1] > 0x14)
    {
      *pIconDim = 0x14;
    }
    wOffs += 84;
    if ((wOffs != 0) && (wOffs < 84))
    {
          /* 64kB overflow */
      wSeg += 0x1000;
      if (wSeg == 0x4000)
      {
            /* correct bank */
        lIcon += 0x4000;
        SwitchBank((((lIcon) >> 16) & 0xFFFE),1);
        wSeg = 0x2000;
      }
      p = MK_FP(wSeg,0);
      far_memcpy(&(pInfo->listicon[84-wOffs]),p,wOffs);
    }

    return 1;
  }

  return 0;
}
