/*
 * AddInInfo.h
 *
 * Author: J. Steingraeber
 * Date: 2001/06/10
 *
 * Released to the public domain
 */


#ifndef AddInInfo_h_included
#define AddInInfo_h_included

#define MAXCOMMENTLENGTH 64

/*
 * AddInInfo
 *
 * structure to receive Add-In data of a specific Add-In
 *
 * this structure is filled from the header of an Add-In
 * by GetAddInInfo
 *
 */
typedef struct tagAddInInfo
{
  char name[16];                /* zero terminated string */
  unsigned long size;           /* length in bytes */
  unsigned long orgsize;        /* original length */
  unsigned int date_year;       /* year of creation */
  unsigned char date_month;     /* month of creation */
  unsigned char date_day;       /* day of creation */
  unsigned char date_hour;      /* hour of creation */
  unsigned char date_minute;    /* minute of creation */
  unsigned int version;         /* version, e.g. 100 means version 1.00 */
  unsigned int lib_year;        /* year of library compiled against */
  unsigned char lib_month;      /* month of library compiled against */
  unsigned char lib_day;        /* day of library compiled against */
  unsigned char lib_hour;       /* hour of library compiled against */ 
  unsigned char lib_minute;     /* minute of library compiled against */
  unsigned int lib_version;     /* library version compiled against */
  unsigned char menuicon[176];  /* ready for LibPutGraph() data 45*28 pixels */
  unsigned char listicon[84];   /* ready for LibPutGraph() data 27*20 pixels */
  unsigned int flags;           /* extra information: deleted, "packed" */
  unsigned long address;        /* linear address of Add-In (for StartAddIn) */
  char comment[MAXCOMMENTLENGTH];           /* comment (entered in PVM) */
} AddInInfo;


/*
 * flags for AddInInfo.flags
 */
#define ADDININFO_DELETED 1 /* "packed" Add-Ins are always marked as deleted */
#define ADDININFO_PACKED 2


/*
 * GetAddInInfo
 *
 * fills an AddInInfo structure with information from
 * requested Add-In iNr
 *
 * iNr is of range 0..63 with regular Add-Ins at 0,4,8,...,60
 * and packed Add-Ins possibly in between
 *
 * returns 1 if Add-In is found and AddInInfo valid
 * returns 0 otherwise, e.g. there is no Add-In starting at the slot
 *
 * The strategy to look for regular Add-Ins is to call GetAddInInfo
 * for slot 0,4,8,...,60 and check the flags if the Add-In is deleted
 *
 * Example:
 *
 * int i;
 * int k, maxsub; // needed for "packed" Add-In search only
 * AddInInfo info;
 *
 * for (i=0;i<64;i+=4)
 * {
 *     if (GetAddInInfo(i,&info) && ((info.flags & ADDININFO_DELETED) == 0))
 *     {
 *         // regular not deleted Add-In found
 *         // ...
 *     }
 * }
 *
 * The strategy to look for "packed" Add-Ins is to examine the
 * size of each not deleted regular Add-In found and test for
 * the next (size / 0x4000) Add-Ins
 *
 * Example:
 *
 * // continuation of above example if regular Add-In was found
 *
 * maxsub = info.size / 0x4000;
 *
 * for (k=1;k<maxsub;++k)
 * {
 *     if (GetAddInInfo(i+k,&info))
 *     {
 *         // "packed" Add-In found
 *         // ADDININFO_DELETED is always set
 *     }
 * }
 *
 */
int GetAddInInfo(int iNr, AddInInfo far * pInfo);

static int k, maxsub; 
static AddInInfo info;
  

/*
 * StartAddIn
 *
 * calls the Add-In at address lLinAddr
 *
 * sets some PVOS internal variables so that PV can savely switch off and on
 *
 * lLinAddr should be taken from AddInInfo
 */
void StartAddIn(unsigned long lLinAddr);


/*
 * CanDelAddIn
 *
 * test if the Add-In number iNr (0..63) could be deleted
 *
 * not deletable are Add-Ins which
 * - belong to PVOS (Mail or PocketSheet)
 * - are "packed"
 * - are currently running
 *
 * Returns 1 if the Add-In could be deleted by DelAddIn()
 * returns 0 otherwise
 */
int CanDelAddIn(int iNr);


/*
 * DelAddIn
 *
 * deletes the Add-In number iNr (0..63)
 *
 * actually this will only mark the Add-In as deleted
 * as it is done by PVOS/PVM
 *
 * some precautions take place to not delete build-in Add-Ins
 * on PV-S[24]50 or PV-750(+)
 *
 * Returns 1 if the Add-In is successfully marked as deleted,
 * returns 0 otherwise
 */
int DelAddIn(int iNr);

/*
 * DefragAddInMem
 *
 * defragments Add-In memory
 *
 * does not return -> LibJmpMenu()
 *
 */
void DefragAddInMem();

#endif
